/*
    $Id: caching_factory.h 269 2011-12-01 10:24:29Z matwey $
    Copyright (C) 2011  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _CACHING_FACTORY_H
#define _CACHING_FACTORY_H

#ifndef GET_OBJECT_MAX_LENGTH
#define GET_OBJECT_MAX_LENGTH 8
#endif

#include <map>

#ifdef HAVE_CONFIG_H
# include "config.h" 
#else
# define HAVE_BOOST_FUNCTIONAL_VALUE_FACTORY_HPP
#endif

#ifdef HAVE_BOOST_FUNCTIONAL_VALUE_FACTORY_HPP
# include <boost/functional/value_factory.hpp>
# define DEFAULT_CREATE_FACTORY_TOKEN =boost::value_factory<T>
#else
# define DEFAULT_CREATE_FACTORY_TOKEN
#endif

#include <boost/preprocessor/arithmetic/inc.hpp>
#include <boost/preprocessor/repetition/enum_binary_params.hpp>
#include <boost/preprocessor/repetition/enum_params.hpp>
#include <boost/preprocessor/repetition/repeat.hpp>

namespace utils {
namespace algo {

template< class T, class Key, class Create DEFAULT_CREATE_FACTORY_TOKEN, class Container = std::map<Key,T> > class caching_factory {
#undef DEFAULT_CREATE_FACTORY_TOKEN
public:
	typedef T       value_type;
	typedef Key     key_type;
	typedef Container container_type;
	typedef Create  creator_type;
	
	typedef typename container_type::const_iterator const_iterator;
private:
	container_type  storage_;
	creator_type    create_;
public:
	explicit caching_factory( Create create = Create() ): create_(create) {
	}

#ifdef GENERATE_GET_OBJECT
#error "GENERATE_GET_OBJECT(z, n, _) is already defined"
#else
#define GENERATE_GET_OBJECT(z, n, _) \
	template<BOOST_PP_ENUM_PARAMS_Z(z, BOOST_PP_INC(n), class T)> \
	const typename container_type::mapped_type& operator() (BOOST_PP_ENUM_BINARY_PARAMS_Z(z, BOOST_PP_INC(n), T, p)) { \
		typename container_type::const_iterator it = storage_.find(Key(BOOST_PP_ENUM_PARAMS_Z(z, BOOST_PP_INC(n), p))); \
		if( it != storage_.end() ) { \
			return it->second; \
		} \
		\
		std::pair< typename container_type::iterator, bool > ret = storage_.insert( std::make_pair(\
			Key(BOOST_PP_ENUM_PARAMS_Z(z, BOOST_PP_INC(n), p)), \
			create_(BOOST_PP_ENUM_PARAMS_Z(z, BOOST_PP_INC(n), p))) ); \
		\
		return ret.first->second; \
	}
	
BOOST_PP_REPEAT(GET_OBJECT_MAX_LENGTH, GENERATE_GET_OBJECT, nil)

#undef GENERATE_GET_OBJECT
#endif
	const const_iterator begin() const { return storage_.begin(); }
	const const_iterator end()   const { return storage_.end(); }
};

} // algo
} // utils

#endif // _CACHING_FACTORY_H
