/*
    $Id: dimm.cpp 327 2024-10-30 12:29:33Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cmath>
#include <functional>

#include "dimm.h"

namespace atmos {
namespace dimm {

vars_type correction( const vars_type& vars ) {
	vars_type ret( vars );

	ret(1) = ret(1)*ret(1) - ret(3)*ret(3); // CCD noise removal
	value_type rho = ret(2)/ret(1);
	ret(1) = 1.15*ret(1) - 0.15*ret(2); // exposure correction
	ret(2) = rho;
	
	return ret;
}

value_type turbulence( value_type scale, value_type Knumber, value_type variance ) {
	value_type ret = variance;
	ret *= scale * scale / 16.7 / Knumber;
	return ret;
}

value_type seeing( value_type turbulence_power ) {
	value_type ret = 2.0e7 * std::pow( turbulence_power, 0.6 );
	return ret;
}

storage::storage( size_type reserve ):
	table_storage< detail::sample >( reserve ),
	m_left_flux_column( *this, std::mem_fn(&detail::sample::left_flux) ),
	m_right_flux_column( *this, std::mem_fn(&detail::sample::right_flux) ),
	m_xvars_column( *this, std::mem_fn(&detail::sample::xvars) ),
	m_yvars_column( *this, std::mem_fn(&detail::sample::yvars) )
	{
}

}}
