/*
    $Id: file_output.h 279 2012-02-23 14:27:56Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _FILE_OUTPUT_H
#define _FILE_OUTPUT_H

#include <iosfwd>
#include <string>

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/optional.hpp>

#include "typesdef.h" // object

namespace atmos {
namespace output {

using namespace boost::numeric::ublas;

typedef boost::posix_time::ptime time_type;
typedef double               value_type;
typedef vector< value_type > vector_type;
typedef symmetric_matrix< value_type > symmetric_matrix_type;

typedef vector_type flux_type;
typedef symmetric_matrix_type indices_type;
typedef vector_type indesis_type;
typedef vector_type cn2_type;
typedef vector_type grid_type;
typedef symmetric_matrix_type residual_type;
typedef object object_type;

class file {
public:
	typedef atmos::output::flux_type flux_type;
	typedef atmos::output::indices_type indices_type;
	typedef atmos::output::indesis_type indesis_type;
	typedef atmos::output::cn2_type cn2_type;
	typedef atmos::output::grid_type grid_type;
	typedef atmos::output::residual_type residual_type;
	typedef atmos::output::object_type object_type;
private:
	std::ostream& m_stm;
public:
	file( std::ostream& stm );
	~file();

	void comment( const std::string& comment );
	void normal( const time_type& time, double mz );
	void background( const time_type& time );
	void atmosphere( const time_type& time, double free_seeing, double free_seeing_rerr, double dimm_seeing, double dimm_seeing_rerr, double dimm_turbulence, double dimm_turbulence_rerr, double isoplanatic_angle, double isoplanatic_angle_rerr, double altitude_eff, double altitude_eff_rerr, double desi_time_constant, double desi_time_constant_rerr, double time_constant, double time_constant_rerr, double moment_0, double moment_0_rerr, double moment_2, double moment_2_rerr );
	void profile( const time_type& time, const std::string& method, double cn2seeing, double chi2, const cn2_type& cn2, const cn2_type& cn2_err, const grid_type& grid );
	void residual( const time_type& time, const std::string& method, const residual_type& residual, const boost::optional<std::pair<double,double> >& residual_dimm = boost::optional<std::pair<double,double> >() );
	void fluxes( const time_type& time, const flux_type& result_flux );
	void fluxes( const time_type& time, const flux_type& result_flux, const flux_type& result_flux_err );
	void indices( const time_type& time, const indices_type& result_indx, const indices_type& result_indx_err, const indesis_type& result_desi, const indesis_type& result_desi_err );
	void parameter( const time_type& time, const std::string& key, const std::string& value );
	void objectinfo( const time_type& time, const object_type& obj );

	std::ostream& operator()() { return m_stm; }
};

}}

#endif // _FILE_OUTPUT_H
