/*
    $Id: gsl_wrap.h 180 2011-01-30 15:56:33Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _GSL_WRAP_H
#define _GSL_WRAP_H

#include <stdexcept>

#include <gsl/gsl_errno.h>
#include <gsl/gsl_integration.h>

namespace gsl {
	class error:
	public std::runtime_error {
	private:
		int gsl_code;
	public:
		error( int gsl_code ):
			runtime_error( gsl_strerror(gsl_code) ),
			gsl_code(gsl_code) { }

		int code() const { return gsl_code; }
	};

	class set_error_handler_off {
	private:
		gsl_error_handler_t * old_handler;
	public:
		set_error_handler_off():
			old_handler( gsl_set_error_handler_off() ) { }
		~set_error_handler_off() {
			gsl_set_error_handler( old_handler );
		}
	};

	class integration_workspace {
	private:
		gsl_integration_workspace * workspace;
	public:
		integration_workspace(const size_t n):
			workspace( gsl_integration_workspace_alloc(n) ) {}
		~integration_workspace() {
			gsl_integration_workspace_free( workspace );
		}
		operator gsl_integration_workspace* () const { return workspace; }
	};

	inline void integration_qagiu (gsl_function * f,
		double a,
		double epsabs, double epsrel, size_t limit,
		gsl_integration_workspace * workspace,
		double *result, double *abserr ) {

		int ret = gsl_integration_qagiu(f,a,epsabs,epsrel,limit,workspace,result,abserr);
		if( ret ) throw error( ret );
	}
	inline void integration_qags (const gsl_function * f,
		double a, double b,
		double epsabs, double epsrel, size_t limit,
		gsl_integration_workspace * workspace, double * result, double * abserr ) {

		int ret = gsl_integration_qags(f,a,b,epsabs,epsrel,limit,workspace,result,abserr);
		if( ret ) throw error( ret );
	}

}

#endif // _GSL_WRAP_H
