/*
    $Id$
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _IO_H
#define _IO_H

#include <iosfwd>
#include <vector>
#include <boost/io/ios_state.hpp>
#include <boost/numeric/ublas/vector.hpp>

#include "typesdef.h"

class ios_indic_saver:
	public boost::io::ios_iword_saver {
public:
	explicit ios_indic_saver( boost::io::ios_iword_saver::state_type& state );
};

namespace std {
	template<class T> std::istream& operator>>( std::istream& stm, std::vector<T>& x ) {
		typedef typename std::vector<T>::value_type value_type;

		value_type tmp;
		while( stm >> tmp ) x.push_back( tmp );
		return stm;
	}
	template<class T, class VA> std::istream& operator>>( std::istream& stm, boost::numeric::ublas::vector<T,VA>& x ) {
		typedef typename boost::numeric::ublas::vector<T,VA>::size_type size_type;

		size_type i = 0;
		while( i < x.size() && stm >> x(i++) );
		return stm;
	}
	template<class T> std::ostream& operator<<( std::ostream& stm, const std::vector<T>& x ) {
		typedef typename std::vector<T>::const_iterator const_iterator;

		const_iterator it = x.begin();
		if( it != x.end() )
			stm << *(it++);
		for( ; it != x.end(); ++it ){
			stm << stm.fill() << *it;
		}
		return stm;
	}
	template<class T, class VA> std::ostream& operator<<( std::ostream& stm, const boost::numeric::ublas::vector<T,VA>& x ) {
		typedef typename boost::numeric::ublas::vector<T,VA>::const_iterator const_iterator;

		const_iterator it = x.begin();
		if( it != x.end() )
			stm << *(it++);
		for( ; it != x.end(); ++it ){
			stm << stm.fill() << *it;
		}
		return stm;
	}
	std::ostream& indic( std::ostream& stm );
	std::ostream& operator<<( std::ostream& stm, const atmos::datum& m );
	std::ostream& operator<<( std::ostream& stm, const atmos::object& obj );
	std::istream& operator>>( std::istream& stm, atmos::object& obj );
}

#endif // _IO_H
