/*
    $Id: utils.cpp 180 2011-01-30 15:56:33Z matwey $
    Copyright (C) 2011  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cmath>

#include "utils.h"

namespace utils {
namespace algo {

linear_generator::linear_generator( result_type start, result_type step ): start_(start), step_(step) {
}
linear_generator::result_type linear_generator::operator()( size_type i ) const {
	return ( start_ + step_ * i );
}
linear_generator::size_type linear_generator::size( result_type max ) const {
	return static_cast<size_type>( std::floor( (max - start_) / step_ ) );
}

lg_generator::lg_generator( result_type start, result_type lstep, result_type gstep ): start_(start), t_(lstep), q_(result_type(1) + gstep / 2) {
	max_ = static_cast< size_type >( std::floor( result_type(2) / gstep - start / lstep ) );
}
lg_generator::result_type lg_generator::operator()( size_type i ) const {
	if( i <= max_ ) return ( start_ + t_ * i );
	return ( start_ + t_ * max_ ) * std::pow( q_, static_cast<int>(i - max_) ); // FIXME: pow(double,int) for <4.2.1
}
lg_generator::size_type lg_generator::size( result_type max ) const {
	result_type gmax = std::floor( std::log( max / ( start_ + t_ * max_ ) ) / std::log( q_ ) );
	if( gmax > result_type(0) ) return static_cast< size_type >( gmax ) + max_ + 1;
	return static_cast< size_type >(std::floor( (max - start_) / t_ ));
}

}}
