/*
    $Id: utils.h 180 2011-01-30 15:56:33Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _UTILS_H_
#define _UTILS_H_

#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/triangular.hpp>
#include <boost/numeric/ublas/vector.hpp>

namespace utils {
namespace ublas {

using namespace boost::numeric::ublas;

template<class ME> matrix_vector_slice< const ME > diag( const ME& x ) {
	typename ME::size_type r = std::min( x.size1(), x.size2() );
	slice s(0, 1, r);
	return matrix_vector_slice< const ME >( x, s, s );
}
template<class ME> matrix_vector_slice< ME > diag( ME& x ){
	typename ME::size_type r = std::min( x.size1(), x.size2() );
	slice s(0, 1, r);
	return matrix_vector_slice< ME >( x, s, s );
}
template<class ME, class TT> triangular_adaptor< const ME, TT > triangle( const ME& x, TT tag ) {
	return triangular_adaptor< const ME, TT >( x );
}
template<class ME, class TT> triangular_adaptor< ME, TT > triangle( ME& x, TT tag ) {
	return triangular_adaptor< ME, TT >( x );
}

template< class F, class V > vector< typename F::result_type > project( const V& grid, F f ) {
	typedef typename F::result_type value_type;
	typedef typename vector< value_type >::size_type size_type;

	vector< value_type > ret( grid.size() );
	for( size_type i = 0; i < ret.size(); ++i )
		ret( i ) = f( grid(i) );
	return ret;
}

template< class T > matrix< T > vector_of_vector_to_matrix( const vector< vector< T > >& v ) {
	typedef T value_type;
	typedef typename matrix< T >::size_type size_type;

	matrix < T > ret( v(0).size(), v.size() );
	for( size_type i = 0; i < ret.size1(); ++i )
		for( size_type j = 0; j < ret.size2(); ++j )
			ret(i,j) = v(j)(i);

	return ret;
}

} // ublas

namespace algo {
struct linear_generator {
	typedef double result_type;
	typedef size_t size_type;

	linear_generator( result_type start, result_type step );
	result_type operator()( size_type i ) const;
	size_type size( result_type max ) const;
private:
	result_type start_,step_;
};
struct lg_generator {
	typedef double result_type;
	typedef size_t size_type;

	lg_generator( result_type start, result_type lstep, result_type gstep );
	result_type operator()( size_type i ) const;
	size_type size( result_type max ) const;
private:
	result_type start_,t_,q_;
	size_type max_;
};
} // algo
} // utils

#endif // _UTILS_H_
