/*
    $Id: moon.cpp 35 2014-03-12 08:49:12Z matwey $
    Copyright (C) 2014  Matwey V. Kornilov <matwey.kornilov@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <yalpa/sidtime.h>
#include <yalpa/moon.h>

#include <yalpa/io.h>

#include <cmath>

yalpa::ecliptic<double> yalpa::moon_position( double julian ) {
	/* from Jean Meeus */
	double t = (julian - 2451545.0) / 36525.0; // Julian centures
	double t2 = t*t;

	double Lp = M_PI / 180.0 * (218.3164591 + 481267.88134236 * t - 0.0013268 * t2 + t2*t / 53841 - t2*t2 / 65194000);
	double D = M_PI / 180.0 * (297.8502042 + 445267.1115168 * t - 0.0016300 * t2 + t2*t / 53841 - t2*t2 / 113065000);
	double M = M_PI / 180.0 * (357.5291092 + 35999.0502909 * t - 0.0001536 * t2 + t2*t / 24490000);
	double Mp = M_PI / 180.0 * (134.9634114 + 477198.8676313 * t + 0.0089970 *t2 + t2*t / 69699 - t2*t2 / 14712000);
	double F = M_PI / 180.0 * (93.2720993 + 483202.0175273 * t - 0.0034029 * t2 - t2*t / 3526000 + t2*t2 / 863310000);

	double dl = M_PI / 180.0 * 1e-6 * (6288774.000000*sin(+1*Mp)
		+1274027.000000*sin(+2*D -1*Mp)
		+658314.000000*sin(+2*D )
		+213618.000000*sin(+2*Mp)
		-185116.000000*sin(+1*M)
		-114332.000000*sin(+2*F)
		+58793.000000*sin(+2*D -2*Mp )
		+57066.000000*sin(+2*D -1*M -1*Mp)
		+53322.000000*sin(+2*D +1*Mp)
		+45758.000000*sin(+2*D -1*M)
		-40923.000000*sin(+1*M -1*Mp)
		-34720.000000*sin(+1*D)
		-30383.000000*sin(+1*M +1*Mp)
		+15327.000000*sin(+2*D -2*F));
	double db = M_PI / 180.0 * 1e-6 * (+5128122.000000*sin(+1*F)
		+280602.000000*sin(+1*Mp +1*F)
		+277693.000000*sin(+1*Mp -1*F)
		+173237.000000*sin(+2*D-1*F)
		+55413.000000*sin(+2*D -1*Mp +1*F)
		+46271.000000*sin(+2*D -1*Mp -1*F)
		+32573.000000*sin(+2*D +1*F)
		+17198.000000*sin(+2*Mp +1*F));

	return yalpa::ecliptic<double>(Lp + dl,db);	
}
yalpa::ecliptic<double> yalpa::moon_position( const std::time_t& epoch ) {
	return moon_position(julian(epoch));
}

