#define BOOST_TEST_MODULE distance_test
#include <boost/test/included/unit_test.hpp>

#include <yalpa/spheric.h>

using namespace yalpa;

BOOST_AUTO_TEST_CASE(distance1) {
	spheric<double> north(0,        M_PI/2);
	spheric<double> east (M_PI/2,   M_PI/2);
	spheric<double> south(M_PI,     M_PI/2);
	spheric<double> west (3*M_PI/2, M_PI/2);
	spheric<double> up  (0, 0);
	spheric<double> down(0, M_PI);

	BOOST_CHECK_SMALL(distance(north, north), 1e-6);
	BOOST_CHECK_CLOSE(distance(north, east), M_PI/2, 1e-6);
	BOOST_CHECK_CLOSE(distance(north, west), M_PI/2, 1e-6);
	BOOST_CHECK_CLOSE(distance(north, south), M_PI, 1e-6);

	BOOST_CHECK_SMALL(distance(up, up), 1e-6);
	BOOST_CHECK_CLOSE(distance(up, north), M_PI/2, 1e-6);
	BOOST_CHECK_CLOSE(distance(up, east), M_PI/2, 1e-6);
	BOOST_CHECK_CLOSE(distance(up, west), M_PI/2, 1e-6);
	BOOST_CHECK_CLOSE(distance(up, south), M_PI/2, 1e-6);
	BOOST_CHECK_CLOSE(distance(up, down), M_PI, 1e-6);
}

BOOST_AUTO_TEST_CASE(distance2) {
	spheric<double> p1(0,   M_PI/2);
	spheric<double> p2(0.5, M_PI/2);
	spheric<double> p3(1.0, M_PI/2);

	BOOST_CHECK_CLOSE(distance(p1, p2), 0.5, 1e-6);
	BOOST_CHECK_CLOSE(distance(p2, p3), 0.5, 1e-6);
	BOOST_CHECK_CLOSE(distance(p1, p3), 1.0, 1e-6);
}

BOOST_AUTO_TEST_CASE(distance3) {
	spheric<double> p1(0, 0);
	spheric<double> p2(0, 0.5);
	spheric<double> p3(0, 1.0);

	BOOST_CHECK_CLOSE(distance(p1, p2), 0.5, 1e-6);
	BOOST_CHECK_CLOSE(distance(p2, p3), 0.5, 1e-6);
	BOOST_CHECK_CLOSE(distance(p1, p3), 1.0, 1e-6);
}

BOOST_AUTO_TEST_CASE(distance4) {
	spheric<double> p1(M_PI/4, M_PI/4);
	spheric<double> p2(2*M_PI/3, M_PI/4);

	BOOST_CHECK_CLOSE(distance(p1, p2), 0.890003222378341, 1e-6);
}
