#define BOOST_TEST_MODULE equatorial_test
#include <boost/test/included/unit_test.hpp>

#include <yalpa/spheric.h>

using namespace yalpa;

BOOST_AUTO_TEST_CASE(equatorial_ha) {
	equatorial<double> eqt(0.5, 0.7);
	BOOST_CHECK_CLOSE(eqt.ha(0),       2*M_PI-eqt.ra(), 1e-6);
	BOOST_CHECK_CLOSE(eqt.ha(2*M_PI),  2*M_PI-eqt.ra(), 1e-6);
	BOOST_CHECK_CLOSE(eqt.ha(-2*M_PI), 2*M_PI-eqt.ra(), 1e-6);
	BOOST_CHECK_CLOSE(eqt.ha(4*M_PI),  2*M_PI-eqt.ra(), 1e-6);
	BOOST_CHECK_SMALL(eqt.ha(0.5),        1e-6);
	BOOST_CHECK_SMALL(eqt.ha(0.5+2*M_PI), 1e-6);
	BOOST_CHECK_SMALL(eqt.ha(0.5-2*M_PI), 1e-6);
}

BOOST_AUTO_TEST_CASE(equatorial_ha2) {
	geographic<double> loc(0.6565346927762977, 0.9730210579868387);
	equatorial<double> eqt1(0.6544984694978736, 1.5533430342749532);
	equatorial<double> eqt2(3.272492347489368, -1.0995574287564276);
	equatorial<double> eqt3(1.0, 0.5*M_PI-0.9730210579868387);
	geographic<double> loc2(1.0, 0.0);

	BOOST_CHECK_EQUAL(eqt1.ha(loc), std::numeric_limits<double>::infinity());
	BOOST_CHECK_EQUAL(eqt2.ha(loc), -std::numeric_limits<double>::infinity());
	BOOST_CHECK_CLOSE(eqt3.ha(loc), M_PI, 1e-6);

	BOOST_CHECK_CLOSE(eqt1.ha(loc2), 0.5*M_PI, 1e-6);
	BOOST_CHECK_CLOSE(eqt2.ha(loc2), 0.5*M_PI, 1e-6);
	BOOST_CHECK_CLOSE(eqt3.ha(loc2), 0.5*M_PI, 1e-6);
}

