/*
    $Id: sun.cpp 32 2013-08-04 15:27:24Z matwey $
    Copyright (C) 2010  Matwey V. Kornilov <matwey.kornilov@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <ctime>
#include <iostream>

#include <yalpa/io.h>
#include <yalpa/sidtime.h>
#include <yalpa/spheric.h>
#include <yalpa/sun.h>

int main( int argc, char** argv ){

	std::time_t now = std::time( NULL );

	std::cout << "Now (UTC):   " << std::asctime( std::gmtime( &now ) ) << std::endl;
	std::cout << "Now (local): " << std::asctime( std::localtime( &now ) ) << std::endl;
	std::cout << "Julian date: " << std::fixed << yalpa::julian( now ) << std::endl;
        std::cout << "GMT siderial:" << std::hours << std::sexagesimal << yalpa::make_angle(yalpa::sidereal_time( now )) << std::endl;

	yalpa::ecliptic<double> sun = yalpa::sun_position( now );
	
	std::cout << "Sun long.:   " << std::degrees << std::sexagesimal << yalpa::make_angle( sun.longitude() ) << std::endl;
	std::cout << std::endl;

	yalpa::equatorial<double> sun_eq( sun );

	std::cout << "Sun ra,dec: " << std::degrees << std::sexagesimal << yalpa::make_angle( sun_eq.ra() ) << " " << std::degrees << yalpa::make_angle( sun_eq.dec() ) << std::endl;

	yalpa::geographic<double> moscow( 0.65798911677777783,0.9730210413888889 );
	std::cout << "Sunset ha:  " << std::degrees << std::sexagesimal << yalpa::make_angle( sun_eq.ha(moscow) ) << std::endl;
	std::cout << std::endl;

	yalpa::horizon<double> sun_hz( make_horizon_helper( sun_eq, moscow, yalpa::sidereal_time(now) ) );

	std::cout << "Sun horizon:" << std::degrees << std::sexagesimal << yalpa::make_angle( sun_hz.azimuth() ) << " " << std::degrees << yalpa::make_angle( sun_hz.coaltitude() ) << std::endl;
	std::cout << std::endl;

	std::time_t sset = yalpa::epoch(yalpa::sunset_time(yalpa::julian(now), moscow));
	std::cout << "Today's Sun set (UTC):   " <<  std::asctime( std::gmtime( &sset ) ) << std::endl;
	std::cout << "Today's Sun set (local): " <<  std::asctime( std::localtime( &sset ) ) << std::endl;
	
	std::time_t srise = yalpa::epoch(yalpa::sunrise_time(yalpa::julian(now), moscow));
	std::cout << "Today's Sun rise (UTC):   " <<  std::asctime( std::gmtime( &srise ) ) << std::endl;
	std::cout << "Today's Sun rise (local): " <<  std::asctime( std::localtime( &srise ) ) << std::endl;
	
	return 0;
}
